#!/usr/bin/python

import os
import sys
import subprocess
import shlex
from shutil import copyfile

from hdputils import install_base_pkg, updateHDPDirectoryScript, config_all_nodes, setHadoopEnvVar, home, hdpScript 
from bdutils import setDirPermission, fileSetKV, fileRemoveKey
from charmhelpers.lib.utils import config_get
from charmhelpers.core.hookenv import log, Hooks, relation_get, open_port, related_units, local_unit, unit_get, relation_set
from charmhelpers.core.host import add_user_to_group

sys.path.insert(0, os.path.join(os.environ['CHARM_DIR'], 'lib'))

from charmhelpers.core import hookenv
def setZKConf():
    log("==> setZKConf","INFO") 
    zkconfdir = os.environ['ZOOKEEPER_CONF_DIR']
    if os.path.islink(zkconfdir):
        os.unlink(zkconfdir)
        os.makedirs( zkconfdir)
    source = os.listdir(ZKConfPath)

    for files in source:
        srcFile = os.path.join(os.path.sep, ZKConfPath, files)
        desFile = os.path.join(os.path.sep, zkconfdir, files)
        copyfile(srcFile, desFile)
    subprocess.call(zoocfgscript)
    
def startZK():
    log ("==> startZK")
    zkenvpath = os.path.join(os.path.sep, 'etc', 'zookeeper', 'conf', 'zookeeper-env.sh')
    zkexport = "export ZOOCFGDIR=/etc/zookeeper/conf"
    zkstartcmd = "/usr/lib/zookeeper/bin/zkServer.sh start >> /var/log/zookeeper/zoo.out 2>&1"
    cmd = shlex.split("su - zookeeper -c 'source {};{};{}'".format(zkenvpath, zkexport, zkstartcmd))
    subprocess.check_output(cmd)

def stopZK():
    log ("==> startZK")
    zkenvpath = os.path.join(os.path.sep, 'etc', 'zookeeper', 'conf', 'zookeeper-env.sh')
    zkexport = "export ZOOCFGDIR=/etc/zookeeper/conf"
    zkstartcmd = "/usr/lib/zookeeper/bin/zkServer.sh stop >> /var/log/zookeeper/zoo.out 2>&1"
    cmd = shlex.split("su - zookeeper -c 'source {};{};{}'".format(zkenvpath, zkexport, zkstartcmd))
    subprocess.check_output(cmd)
    
def restartZK():
    log ("==> startZK")
    zkenvpath = os.path.join(os.path.sep, 'etc', 'zookeeper', 'conf', 'zookeeper-env.sh')
    zkexport = "export ZOOCFGDIR=/etc/zookeeper/conf"
    zkstartcmd = "/usr/lib/zookeeper/bin/zkServer.sh restart >> /var/log/zookeeper/zoo.out 2>&1"
    cmd = shlex.split("su - zookeeper -c 'source {};{};{}'".format(zkenvpath, zkexport, zkstartcmd))
    subprocess.check_output(cmd)
    
def getid(unitID):
    return unitID.split("/")[1]
  
def config_zk_nodes():
    log("=> config_zk_nodes")
    updateHDPDirectoryScript("ZOOKEEPER_DATA_DIR=", "\""+config_get('zookeeper_data_dir')+"\";\n")
    config_all_nodes()
    setHadoopEnvVar()
    cmd = "usermod zookeeper -s /bin/bash"
    subprocess.call(cmd.split())
    cmd ='groupadd '+os.environ['HADOOP_GROUP']
    subprocess.call(cmd.split())
    add_user_to_group(os.environ['ZOOKEEPER_USER'], os.environ['HADOOP_GROUP'])
    log_path = os.environ['ZOOKEEPER_LOG_DIR']
    pid_path = os.environ['ZOOKEEPER_PID_DIR']
    data_path = os.environ['ZOOKEEPER_DATA_DIR']
    owner = os.environ['ZOOKEEPER_USER']
    group = os.environ['HADOOP_GROUP']
    myid = os.path.join(os.path.sep, data_path, "myid")
    setDirPermission(log_path, owner, group, 0755)
    setDirPermission(pid_path, owner, group, 0755)
    setDirPermission(data_path, owner, group, 0755)
    with open(myid, 'w+') as df:
        df.writelines(getid(local_unit()))
    zkhost = unit_get('private-address')
    fileSetKV(zoocfg, "dataDir=", data_path)
    # cleaning up zookeeper config
    fileRemoveKey(zoocfg, "server.1=")
    fileSetKV(zoocfg, "server"+"."+getid(local_unit())+"=", zkhost+":2888:3888")
    setZKConf()
    
     

ZKConfPath = os.path.join(os.path.sep,home, hdpScript, "configuration_files", "zookeeper")
zoocfg = os.path.join(os.path.sep, ZKConfPath, "zoo.cfg")
zoocfgscript = os.path.join(os.path.sep, os.environ['CHARM_DIR'], os.path.sep, os.environ['CHARM_DIR'], 'files', 'scripts',"createZKConfDir.sh")

hooks = Hooks()
log = hookenv.log
SERVICE = 'hdp-zookeeper'

@hooks.hook('install')
def install():
    log("=> install zookeeper")
    packages =['ntp', 'openjdk-7-jdk',"zookeeper"]
    install_base_pkg(packages)
    config_zk_nodes()
    open_port(config_get('zk-port'))
    open_port(2888)
    open_port(3888)
    


@hooks.hook('config-changed')
def config_changed():
    log('==> config_changed hdp-zookeeper')



@hooks.hook('upgrade-charm')
def upgrade_charm():
    log('==> Upgrading hdp-zookeeper')


@hooks.hook('start')
def start():
    log('==> start hdp-zookeeper')
    startZK()


@hooks.hook('stop')
def stop():
    log('==> stop hdp-zookeeper')
    stopZK()

@hooks.hook('quorum-relation-changed')
def quorum_relation_changed():
    log("==> Configuring quorum, local ID = {} - Changed phase".format(local_unit()),"INFO")
    setHadoopEnvVar()
    nodeList = related_units()
    for n in nodeList:
        unitIP = relation_get('private-address', unit=n)
        zk = os.path.join(os.path.sep, os.environ['ZOOKEEPER_CONF_DIR'], 'zoo.cfg')
        fileSetKV(zk, "server."+getid(n)+"=", unitIP +":2888:3888")
    restartZK()
    
@hooks.hook('quorum-relation-departed')
def quorum_relation_departed():
    log("==> Configuring quorum departed, local ID = {} - Changed phase".format(local_unit()),"INFO")
    setHadoopEnvVar()
    nodeList = related_units()
    for n in nodeList:
        zk = os.path.join(os.path.sep, os.environ['ZOOKEEPER_CONF_DIR'], 'zoo.cfg')
        fileRemoveKey(zk, "server."+getid(n)+"=")
    restartZK()
            
@hooks.hook('zookeeper-relation-joined')
def zookeeper_relation_joined():
    log('==> zookeeper_relation_joined')
    relation_set(port=config_get('zk-port'))
    
if __name__ == "__main__":
    # execute a hook based on the name the program is called by
    hooks.execute(sys.argv)
